/*******************************************************************************
* Title                 :   XBee Demo
* Filename              :   main.c
* Author                :   RBL
* Origin Date           :   26/11/2015
* Notes                 :   Initial demo for XBee click
*******************************************************************************/
/*************** MODULE REVISION LOG ******************************************
*
*    Date    Software Version    Initials   Description
*  26/11/15           .1         RBL       Module Created.
*
*******************************************************************************/
/**
 * @file main.c
 * @brief This module contains the demo for XBee click
 */
/******************************************************************************
* Includes
*******************************************************************************/
#include <stdint.h>
#include <stdbool.h>
#include <stddef.h>
#include "resources.h"

/******************************************************************************
* Module Preprocessor Constants
*******************************************************************************/
#define MAX_BUFFER_SIZE 200
#define FIELD_SIZE 10

/******************************************************************************
* Module Preprocessor Macros
*******************************************************************************/

/******************************************************************************
* Module Typedefs
*******************************************************************************/

/******************************************************************************
* Module Variable Definitions
*******************************************************************************/
// TFT module connections
unsigned int TFT_DataPort at GPIOE_ODR;
sbit TFT_RST  at GPIOE_ODR.B8;
sbit TFT_RS   at GPIOE_ODR.B12;
sbit TFT_CS   at GPIOE_ODR.B15;
sbit TFT_RD   at GPIOE_ODR.B10;
sbit TFT_WR   at GPIOE_ODR.B11;
sbit TFT_BLED at GPIOE_ODR.B9;
// End TFT module connections

sbit XBEE_RST at GPIOC_ODR.B2;

static char volatile rx_buffer[MAX_BUFFER_SIZE];
static uint8_t volatile position;
static bool volatile packet_flag;

// Colors
uint16_t BACKG_COLOR;

/******************************************************************************
* Function Prototypes
*******************************************************************************/
void system_init( void );
void parse_packet( char *data_in, char data_out[][FIELD_SIZE], uint8_t *num_fields );
void update_display( char tokens[][FIELD_SIZE], uint8_t token_count );
long map( long x, long in_min, long in_max, long out_min, long out_max );

/******************************************************************************
* Function Definitions
*******************************************************************************/
long map(long x, long in_min, long in_max, long out_min, long out_max)
{
  return (x - in_min) * (out_max - out_min) / (in_max - in_min) + out_min;
}


void parse_packet( char *data_in, char data_out[][FIELD_SIZE], uint8_t *num_fields )
{
    char *ptr;
    char *tokens = ",";

    *num_fields = 0;
    ptr = strtok( data_in, tokens );

    while( ptr )
    {
        strcpy( data_out[ ( *num_fields )++ ], ptr );
        ptr = strtok( 0, tokens );
    }

    packet_flag = false;
}

void update_display( char tokens[][FIELD_SIZE], uint8_t token_count )
{
    char tmp_txt[80];
    float temp;
    float humidity;
    float pressure;
    int light;
    int i;
    
    TFT_Set_Pen( BACKG_COLOR, 2 );
    TFT_Set_Brush( 1, CL_BLACK, 0, LEFT_TO_RIGHT, CL_BLACK, BACKG_COLOR );
    //TFT_Set_Brush( 1, BACKG_COLOR, 0, LEFT_TO_RIGHT, CL_BLACK, BACKG_COLOR );
    TFT_Rectangle( 62, 0, 120, 33 );
    TFT_Set_Brush( 1, CL_BLACK, 0, LEFT_TO_RIGHT, CL_BLACK, BACKG_COLOR );
    TFT_Rectangle( 160, 180, 195, 198 );
    TFT_Rectangle( 235, 0, 290, 18 );
    TFT_Set_Pen( CL_WHITE, 1 );

    for( i = 0; i < token_count; i++ )
    {
        switch( tokens[i][0] )
        {
           case 'T':
              temp = atof( &tokens[i][2] );
           break;
           case 'P':
              pressure = atof( &tokens[i][2] );
           break;
           case 'H':
              humidity = atof( &tokens[i][2] );
           break;
           case 'L':
              light = atoi( &tokens[i][2] );
           break;
        }
    }
    
    // Draw values read from sensor
    sprintf( tmp_txt, "%2.0f C", temp );
    TFT_Write_Text( tmp_txt, 165, 180 );
    sprintf( tmp_txt, "%4.0f mb", pressure );
    TFT_Write_Text( tmp_txt, 65, 0 );
    sprintf( tmp_txt, "%2.0f p", humidity );
    TFT_Write_Text( tmp_txt, 65, 14 );
    sprintf( tmp_txt, "%d lux", light );
    TFT_Write_Text( tmp_txt, 240, 0 );
    
    //TFT_Rectangle( 10, 200, 310, 230 );
    //TFT_Rectangle( 290, 10, 310, 190 );
    // White boxes
    TFT_Set_Brush( 1, CL_WHITE, 0, LEFT_TO_RIGHT, CL_BLACK, CL_WHITE);
    TFT_Rectangle( 290, 10, 310, map( light, 0, 100, 190, 10 ) );
    TFT_Rectangle( map( temp, -30, 45, 10, 310 ), 200, 310, 230 );
    
    // Color boxes
    TFT_Set_Brush( 1, CL_YELLOW, 0, LEFT_TO_RIGHT, CL_BLACK, CL_WHITE);
    TFT_Rectangle( 290, map( light, 0, 100, 190, 10 ), 310, 190 );
    TFT_Set_Brush( 1, CL_YELLOW, 1, LEFT_TO_RIGHT, CL_BLUE, CL_RED );
    TFT_Rectangle( 10, 200, map( temp, -30, 45, 10, 310 ), 230 );
    
    if( pressure > 1025 )
        TFT_Image( ( 320 / 2 ) - ( 100 / 2 ), ( 240 / 2 ) - ( 100 / 2 ), sunny, 1 );
    else if( pressure < 1025 )
        TFT_Image( ( 320 / 2 ) - ( 100 / 2 ), ( 240 / 2 ) - ( 100 / 2 ), rainy, 1 );
}

void system_init()
{
    DisableInterrupts();
    
    GPIO_Digital_Output( &GPIOC_BASE, _GPIO_PINMASK_2 );
    
    XBEE_RST = 1;
    
    // Debugging UART part
    UART1_Init_Advanced( 57600,
                         _UART_8_BIT_DATA,
                         _UART_NOPARITY,
                         _UART_ONE_STOPBIT,
                         &_GPIO_MODULE_USART1_PA9_10 );

    // XBee port
    UART3_Init_Advanced( 9600,
                         _UART_8_BIT_DATA, 
                         _UART_NOPARITY,
                         _UART_ONE_STOPBIT, 
                         &_GPIO_MODULE_USART3_PD89 );
                         
    RXNEIE_USART3_CR1_bit = 1;
    Delay_ms( 100 ); // Allow time to stablize
    UART1_Write_Text( "Ready" );

    BACKG_COLOR = TFT_RGBToColor16bit( 20, 129, 203 );
    TFT_Init_ILI9341_8bit( 320, 240 );
    TFT_Fill_Screen( BACKG_COLOR );
    TFT_Set_Pen( CL_WHITE, 1 );
    TFT_Set_Brush( 1, CL_WHITE, 0, LEFT_TO_RIGHT, CL_BLACK, CL_WHITE);
    TFT_Set_Font( TFT_defaultFont, CL_WHITE, FO_HORIZONTAL );
    TFT_Write_Text( "Temp:", 120, 180 );
    TFT_Write_Text( "Pressure:", 0, 0 );
    TFT_Write_Text( "Humidity:", 0, 14 );
    TFT_Write_Text( "Light:", 200, 0 );
    TFT_Rectangle( 10, 200, 310, 230 );
    TFT_Rectangle( 290, 10, 310, 190 );
    // Enable interrupts on incoming xbee transmissions
    NVIC_IntEnable( IVT_INT_USART3 );
}

void main()
{
    char tokens[5][10];
    uint8_t token_count = 0;

    system_init();
    EnableInterrupts();
    
    while( 1 )
    {
        if( packet_flag )
        {
            parse_packet( rx_buffer, tokens, &token_count );
            update_display( tokens, token_count );
        }
    }
}

void UART3_RX_ISR() iv IVT_INT_USART3 ics ICS_AUTO
{
    if( RXNE_USART3_SR_bit )
    {
        char tmp = USART3_DR;

        if( tmp == '\n' )
        {
            packet_flag = true;
            rx_buffer[position] = '\0';
            position = 0;
        } else {
           if( position == MAX_BUFFER_SIZE - 1 ) position = 0;
           UART1_Write( tmp );
           rx_buffer[position++] = tmp;
        }
    }
}
